package main

import (
	"fmt"
	"net/http"
	"path"
	"time"

	"srs.epita.fr/fic-server/dashboard/api"
	"srs.epita.fr/fic-server/settings"

	"github.com/julienschmidt/httprouter"
)

func init() {
	api.Router().GET("/", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, "index.html"))
	})

	api.Router().GET("/css/*_", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, r.URL.Path))
	})
	api.Router().GET("/fonts/*_", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, r.URL.Path))
	})
	api.Router().GET("/img/*_", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, r.URL.Path))
	})
	api.Router().GET("/js/*_", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, r.URL.Path))
	})
	api.Router().GET("/views/*_", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		http.ServeFile(w, r, path.Join(StaticDir, r.URL.Path))
	})

	api.Router().GET("/events.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(TeamsDir, "events.json"))
	})
	api.Router().GET("/my.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(TeamsDir, "public", "my.json"))
	})
	api.Router().GET("/stats.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(TeamsDir, "public", "stats.json"))
	})
	api.Router().GET("/settings.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("X-FIC-Time", fmt.Sprintf("%f", float64(time.Now().UnixNano()/1000)/1000000))
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(settings.SettingsDir, settings.SettingsFile))
	})
	api.Router().GET("/teams.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(TeamsDir, "teams.json"))
	})
	api.Router().GET("/themes.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(TeamsDir, "themes.json"))
	})

	api.Router().GET("/public.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public.json"))
	})
	api.Router().GET("/public0.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public0.json"))
	})
	api.Router().GET("/public1.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public1.json"))
	})
	api.Router().GET("/public2.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public2.json"))
	})
	api.Router().GET("/public3.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public3.json"))
	})
	api.Router().GET("/public4.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public4.json"))
	})
	api.Router().GET("/public5.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public5.json"))
	})
	api.Router().GET("/public6.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public6.json"))
	})
	api.Router().GET("/public7.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public7.json"))
	})
	api.Router().GET("/public8.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public8.json"))
	})
	api.Router().GET("/public9.json", func(w http.ResponseWriter, r *http.Request, ps httprouter.Params) {
		w.Header().Set("Cache-Control", "no-cache")
		http.ServeFile(w, r, path.Join(DashboardDir, "public9.json"))
	})
}
